import 'package:analyzer/dart/analysis/session.dart';
import 'package:analyzer/dart/ast/ast.dart';
import 'package:analyzer/dart/ast/token.dart';
import 'package:analyzer/dart/element/element.dart';
import 'package:build/build.dart';

String lowerTitleCase(String value) {
  return '${value[0].toLowerCase()}${value.substring(1)}';
}

// TODO: This is pointless to me as I don't know how to apply
// the type to the collection using the annotations typeValue
//
// Collection parseCollection(ConstantReader annotation) {
//   final nameField = annotation.read('name');
//   final type = annotation.typeValue;

//   return Collection(name: nameField.stringValue);
// }

Future<AstNode> tryGetAstNodeForElement(
  Element element,
  BuildStep buildStep,
) async {
  var library = element.library!;

  while (true) {
    try {
      final elementDeclaration = library.session
          .getParsedLibraryByElement(library)
          .getElementDeclaration(element);

      if (elementDeclaration == null) {
        throw 'Element Decleration as null';
      }

      return elementDeclaration.node;
    } on InconsistentAnalysisException {
      library = await buildStep.resolver.libraryFor(
        await buildStep.resolver.assetIdForElement(element.library!),
      );
    }
  }
}

List<Token> getGenericTypes(AstNode node) {
  final typeSlice = <Token>[];

  Token? currentToken = node.endToken;
  var insideTypeDeclaration = false;

  while (currentToken != null) {
    if (currentToken.charOffset < node.offset) {
      break;
    }

    if (currentToken.lexeme == '>') {
      insideTypeDeclaration = true;
    } else if (currentToken.lexeme == '<') {
      insideTypeDeclaration = false;
    } else if (insideTypeDeclaration) {
      if (currentToken.isIdentifier) {
        typeSlice.add(currentToken);
      }
    }

    currentToken = currentToken.previous;
  }

  return typeSlice.reversed.toList();
}
